#!/usr/bin/perl -w

use strict;
use Statistics::Descriptive;
use Math::CDF;
use Data::Dumper;

#This program finds the networks for given gene list

print "Please enter the name of the geneList file from VEGAS: ";
my $input_file1 = <STDIN>;
chomp $input_file1;

print "Please enter name of the output file: ";
my $output_file = <STDIN>;
chomp $output_file;

my $starttime = time;

open(IN,$input_file1)or die "\nCould not open input file\n";
open(IN2,"ranksOfGenesInBiogridNetworks.txt")or die "\nCould not open BIOGRID gene ranks file file\n";
open(OUT,">$output_file") or die "\nCould not open output file\n";
print OUT "NetworkNum	TotalGenesInNetwork	TotalSigGenes	TotalNonSigGenes	SigGenes	NonSigGenes	WeightedZscore	NetworkP-value	corrP\n";
my @pvals;
my %sigGenes;
my %nonSigGenes;
while(<IN>)
	{
		my $line = $_;
		$line =~ s/\"//g;
		unless($line =~ /^\d/) {next;}
		my @input = split(/\s+/,$line);
		my $genePval = $input[7];
		if($genePval <= 0.05)
			{
				my $sigGeneName = $input[1];
				my $sigPval = $genePval;
				$sigGenes{$sigGeneName} = $sigPval;
				push(@pvals,$sigPval);
			}
		else
			{
				my $nonSigGeneName = $input[1];
				my $nonSigPval = $genePval;
				$nonSigGenes {$nonSigGeneName} = $nonSigPval;
				push(@pvals,$nonSigPval);
			}
		
	}
close IN;

#calculate mean and standard deviation
my $stat = Statistics::Descriptive::Full->new();
$stat->add_data(@pvals);
my $mean = $stat->mean();
my $std  = $stat->standard_deviation();

#calculate zscore of significant genes
my %sigGenesZscores;
foreach my $key(sort (keys %sigGenes))
	{
		my $sigZscore = (($sigGenes{$key} - $mean)/$std); #z = x-mu/std, mean and stddev are calculated
		$sigGenesZscores{$key} = $sigZscore;
	}

#calculate zscore of non-significant genes
my %nonSigGenesZscores;
foreach my $key1(sort (keys %nonSigGenes))
	{
		my $nonSigZscore = (($nonSigGenes{$key1} - $mean)/$std);
		$nonSigGenesZscores{$key1} = $nonSigZscore;
	}

#Read the networks and genes along with modified Google Pagerank values
my %geneRankPerNet;
while(<IN2>)
	{
		my $line3 = $_;
		my @input3 = split(/\s+/,$line3);
		my $networkName = $input3[0];
		my $geneName = $input3[1];
		my $geneRank = $input3[2];
		$geneRankPerNet{$networkName}{$geneName} = $geneRank;
	}
#print Dumper \%geneRankPerNet;

#calculate the zCombined for each network
foreach my $netKey(sort (keys %geneRankPerNet))
	{
		my $totalGenesPerNetwork = 0;
		my $numerator = 0;
		my $denom = 0;
		my @finalSigGenes = ();
		my @finalNonSigGenes = ();
		foreach my $geneKey(sort (keys %{$geneRankPerNet{$netKey}}))
			{
				$totalGenesPerNetwork = $totalGenesPerNetwork+1;
				if(exists $sigGenesZscores{$geneKey})
					{
						my $geneNumerator = $geneRankPerNet{$netKey}{$geneKey} * $sigGenesZscores{$geneKey};
						$numerator = $numerator + abs($geneNumerator);
						my $weightSquare = $geneRankPerNet{$netKey}{$geneKey} * $geneRankPerNet{$netKey}{$geneKey};
						$denom = $denom + $weightSquare;
						push(@finalSigGenes,$geneKey);
					}
				if(exists $nonSigGenesZscores{$geneKey})
					{
						my $nonSigGeneNumerator = $geneRankPerNet{$netKey}{$geneKey} * $nonSigGenesZscores{$geneKey};
						$numerator = $numerator + abs($nonSigGeneNumerator);
						my $nonSigWeightSquare = $geneRankPerNet{$netKey}{$geneKey} * $geneRankPerNet{$netKey}{$geneKey};
						$denom = $denom + $nonSigWeightSquare;
						push(@finalNonSigGenes,$geneKey);
					}
			}
			my $finalSigLength = scalar(@finalSigGenes);
			my $finalNonSigLength = scalar(@finalNonSigGenes);
			if(($denom eq 0) || ($numerator eq 0))
				{
					next;
				}
			else
				{
					my $zComb = ($numerator/(sqrt($denom)));
					my $zCombPval = Math::CDF::pnorm(-$zComb);#this is 1-pnorm(z)
					#Bonferroni correction for number of genes in the network and total number of networks (which is 2849)
					my $corrP = $zCombPval*($totalGenesPerNetwork+2849);
					if($corrP < 0.05)
						{
							print OUT "$netKey\t$totalGenesPerNetwork\t$finalSigLength\t$finalNonSigLength\t@finalSigGenes\t@finalNonSigGenes\t$zComb\t$zCombPval\t$corrP\n";
						}
				}

	}
my $endtime = time;
my $totaltime = $endtime - $starttime;
print "The total run time of this program in seconds is: $totaltime\n";
close OUT;
close IN;
